function state = loadStateFromTSPLIB( name, num_robots, num_tool_racks, num_task_types )
    disp(name);
    task_cost = 600;
    rack_cost = 1200;
    N = 0;    % Every Nth node (starting from 1) is a robot location
    M = 0;   % Every Mth node (starting from 2) is a tool rack
    if contains(name, 'Pr76', 'IgnoreCase', true)
        N = 8;
        M = 8;
    end
    if contains(name, 'Pr152', 'IgnoreCase', true)
        N = 16;
        M = 16;
    end
    if contains(name, 'Pr226', 'IgnoreCase', true)
        N = 24;
        M = 24;
    end

    
    robot_interval = N;  % starting from 1
    rack_interval = M;   % starting from 2
    
    fid = fopen(name, 'r');
    C = textscan(fid,'%f %f %f', 'HeaderLines', 6);
    fclose(fid);
    
    x = C{2};
    y = C{3};
    % First few are robot positions.
%     num_tasks = length(x) - num_robots;
%     indices = randperm(length(x));
%     robot_indices = indices(1:num_robots);
%     task_indices = sort(indices(num_robots+1:end));
    % start at same spot

%     robot_indices = ones(num_robots,1);
    % Get the indices.
    robot_indices = 1:robot_interval:length(x);
    rack_indices = 2:rack_interval:length(x);
    task_indices = 1:length(x);
    task_indices([robot_indices rack_indices]) = [];
    % If a clash, robot takes priority.
    common = intersect(robot_indices, rack_indices);
    rack_indices = setxor(rack_indices, common);
        
    max_robots = length(robot_indices);
    max_racks = length(rack_indices);
    % Only use the amount we need.
    k = subIndices(max_robots, num_robots);
    robot_indices = robot_indices(k);
    k = subIndices(max_racks, num_tool_racks);
    rack_indices = rack_indices(k);
    
    
    num_tasks = length(task_indices);
    num_racks = length(rack_indices);
    
    disp(['Num Tasks:  ' num2str(num_tasks)]);
    disp(['Num Robots: ' num2str(num_robots) ' out of ' num2str(max_robots)]);
    disp(['Num Racks:  ' num2str(num_racks)  ' out of ' num2str(max_racks) ]);
    
    task_types = (1:num_tasks)';
    task_types = mod(task_types, num_task_types) + 1;
    
    preallocate = 500;
    state = State();
    state.robot_list = [(1:num_robots)' x(robot_indices) y(robot_indices)];
    state.task_list = [(1:num_tasks)' task_types x(task_indices) y(task_indices) ones(num_tasks,1).*task_cost];
%     state.robot_expertise = ones(num_robots,1);
    state.robot_expertise = zeros(num_robots,num_task_types);
%     state.tool_expertise = [zeros(1, num_task_types); eye(num_task_types)];
    state.tool_expertise = [zeros(1, num_task_types); max(eye(num_task_types), 0.1*ones(num_task_types))];
    
    state.return_home = true;
    
    state.tool_rack_list = [ (1:num_racks)' x(rack_indices) y(rack_indices) ones(num_racks,1).*rack_cost ];
    
    
    
    
    state.populateTravelCosts();
    
    state.state_index = 1;
    state.allocated_tasks    = cell(preallocate, 1);
    state.allocated_tasks{1} = [];
    state.unallocated_tasks    = cell(preallocate, 1);
    state.unallocated_tasks{1} = state.task_list(:,1);
    state.path_plan    = cell(preallocate, 1);
    state.path_plan{1} = cell( size(state.robot_list, 1), 1 );   % empty path plans
    
    function indices = subIndices(total, num)
        a = (total+1)/(num+1) * (1:num)';
        % Round to odd
        Z = rem(a,2);
        Z(abs(Z)~=1.5) = 0;
        b = round(a-Z/3);
        indices = b;
    end
    
end