%% Nick Sullivan 2017, The University of Adelaide
% This function performs a sequential single-item auction to solve a
% multiple travelling salesman problem where tasks require tools. 
function [result, result_with_2_opt] = performOptimal( scenario_num )
%% USER OPTIONS
display_graphs = false;
objective = 1;          % 1 - minisum (LOCKED IN)
                        % 2 - minimax
return_home = true;     % if robots have to return to their start point.(LOCKED IN)

transform = true;       % if the system should be transform to a standard MTSP first (LOCKED IN)

num_robots = 3;   
num_task_types = 5;
num_tasks_per_type = 10;
num_tool_racks = 1;

%% Set the scenario
addpath Simulator
rng(scenario_num);
s = State();
s.return_home = return_home;

s.robot_list = [[1:num_robots]' rand(num_robots,2)*100];

task_num = 1;
s.task_list = [];
for i = 1:num_task_types
    for j = 1:num_tasks_per_type
        task = [task_num i 100*rand 100*rand 10];
        task_num = task_num + 1;
        s.task_list = [s.task_list; task];
    end
end

tools = zeros(num_task_types);  % one or the other
% tools = rand(num_task_types); % one or the other
tools = max(tools, eye(num_task_types));
s.tool_expertise = [zeros(1, num_task_types); tools];

s.robot_expertise = zeros(num_robots, num_task_types); % use tools
% s.robot_expertise = ones(num_robots, num_task_types); % use innate xp




% s.tool_rack_list = [1 2 2 1  % [id x y cost] the location of the tool racks
%                     2 8 2 1
%                     3 8 8 1
%                     4 2 8 1];
s.tool_rack_list = [ ones(num_tool_racks,1), rand(num_tool_racks,2)*100, 10*ones(num_tool_racks,1) ];
                    
% s.tool_rack_list = [1 -2 -2 1];
% s.tool_rack_list = [1 40 40 20  % [id x y cost] the location of the tool racks
%                     2 40 40 20
%                     3 160 160 20
%                     4 160 160 20];              
                
s.populateTravelCosts();

% Transform the state into a standard M-TSP problem:
if transform
    s = State.transformToMTSP(s);
end


%% Solve the state
% Clear previous results.
!bash clear_htsp &
    
% Export the scenario to text.
export_folder = 'HTSPOptimalSolution/Instances/';
export_file = ['scenario' num2str(1) '-' num2str(scenario_num)]; 
s.exportScenario([export_folder export_file]);
cost_folder = 'HTSPOptimalSolution/RSInstances/Costs/';
cost_file = ['scenario' num2str(1) '-' num2str(scenario_num)];
s.exportCosts([cost_folder cost_file]);

% Run the optimal solution.
eval(['!bash run_htsp scenario' num2str(1) '-' num2str(scenario_num)]);
 % Get results.
% dists = csvread(['HTSPOptimalSolution/Distances/EdgeResults/' export_file]);
% results = [max(dists) sum(dists) 0 ];
solution_folder = 'HTSPOptimalSolution/Logs/';
fid = fopen([solution_folder export_file]);
while ~feof(fid)
   line = fgets(fid);
   if contains(line, 'Found incumbent of value') 
       str = line(1:strfind(line, 'after'));
       num = str2num( str(~isletter(str) ));
%       disp(['line: ' str]); 
%       disp(['num: ' num2str( num)]);
   end
end


result = num + num_task_types*num_tasks_per_type; % if task completion cost is 1
result_with_2_opt = result;

%% Convert solution to state for plotting.
    


end
